@@ -35,6 +35,9 @@ RS_DEFAULT_STRONG_LEN = 8
 RS_DEFAULT_BLOCK_LEN = 2048
 
 
+# DEFINES FROM librsync.h:
+#-------------------------
+
 # librsync.h: rs_buffers_s
 class Buffer(ctypes.Structure):
     _fields_ = [
@@ -46,8 +49,47 @@ class Buffer(ctypes.Structure):
         ('avail_out', ctypes.c_size_t),
     ]
 
+# char const *rs_strerror(rs_result r);
 _librsync.rs_strerror.restype = ctypes.c_char_p
+_librsync.rs_strerror.argtypes = (ctypes.c_int, )
 
+# rs_job_t *rs_sig_begin(size_t new_block_len, size_t strong_sum_len);
+_librsync.rs_sig_begin.restype = ctypes.c_void_p
+_librsync.rs_sig_begin.argtypes = (ctypes.c_size_t, ctypes.c_size_t, )
+
+# rs_job_t *rs_loadsig_begin(rs_signature_t **);
+_librsync.rs_loadsig_begin.restype = ctypes.c_void_p
+_librsync.rs_loadsig_begin.argtypes = (ctypes.c_void_p, )
+
+# rs_job_t *rs_delta_begin(rs_signature_t *);
+_librsync.rs_delta_begin.restype = ctypes.c_void_p
+_librsync.rs_delta_begin.argtypes = (ctypes.c_void_p, )
+
+# rs_job_t *rs_patch_begin(rs_copy_cb *, void *copy_arg);
+_librsync.rs_patch_begin.restype = ctypes.c_void_p
+_librsync.rs_patch_begin.argtypes = (ctypes.c_void_p, ctypes.c_void_p, )
+
+# rs_result rs_build_hash_table(rs_signature_t* sums);
+_librsync.rs_build_hash_table.restype = ctypes.c_size_t
+_librsync.rs_build_hash_table.argtypes = (ctypes.c_void_p, )
+
+# rs_result rs_job_iter(rs_job_t *, rs_buffers_t *);
+_librsync.rs_job_iter.restype = ctypes.c_int
+_librsync.rs_job_iter.argtypes = (ctypes.c_void_p, ctypes.c_void_p, )
+
+# void rs_trace_set_level(rs_loglevel level);
+_librsync.rs_trace_set_level.restype = None
+_librsync.rs_trace_set_level.argtypes = (ctypes.c_int, )
+
+# void rs_free_sumset(rs_signature_t *);
+_librsync.rs_free_sumset.restype = None
+_librsync.rs_free_sumset.argtypes = (ctypes.c_void_p, )
+
+# rs_result rs_job_free(rs_job_t *);
+_librsync.rs_job_free.restype = ctypes.c_int
+_librsync.rs_job_free.argtypes = (ctypes.c_void_p, )
+
+# A function declaration for our read callback.
 patch_callback = ctypes.CFUNCTYPE(ctypes.c_void_p, ctypes.c_int, ctypes.c_size_t,
                                   ctypes.POINTER(Buffer))
 
@@ -82,14 +124,23 @@ def _execute(job, f, o=None):
         if result == RS_DONE:
             break
         elif result != RS_BLOCKED:
-            # TODO: I don't think error reporting works properly.
             raise LibrsyncError(result)
+        if buff.avail_in > 0:
+            # There is data left in the input buffer, librsync did not consume
+            # all of it. Rewind the file a bit so we include that data in our
+            # next read. It would be better to simply tack data to the end of
+            # this buffer, but that is very difficult in Python.
+            f.seek(f.tell() - buff.avail_in)
     if o and callable(getattr(o, 'seek', None)):
         # As a matter of convenience, rewind the output file.
         o.seek(0)
     return o
 
 
+def debug(level=7):
+    _librsync.rs_trace_set_level(level)
+
+
 def signature(f, s=None, block_size=RS_DEFAULT_BLOCK_LEN):
     """
     Generate a signature for the file `f`. The signature will be written to `s`.
@@ -147,9 +198,9 @@ def patch(f, d, o=None):
         o = tempfile.SpooledTemporaryFile(max_size=MAX_SPOOL)
 
     @patch_callback
-    def read_cb(opaque, pos, len, buff):
+    def read_cb(pos, length, buff):
         f.seek(pos)
-        block = f.read(len)
+        block = f.read(length)
         buff.next_in = ctypes.c_char_p(block)
         buff.avail_in = ctypes.c_size_t(len(block))
 
